# =============================================================================================
# Minimax Estimation of the Volume of a Set under the Rolling Ball Condition
# =============================================================================================

# Numerical experiments (Section 4)
# ---------------------------------------

# Convex case
# Comparison with the results in:
# Unbiased estimation of the volume of a convex body. Nikolay Baldin, Markus Reib (2016)
# Stochastic Processes and their Applications 126 (12), 3716 - 3732.


library(alphahull)   # Requires alphahull package (version 2.1)
source("volest.R")   # Read in R the function volest (minimax estimation of the volume)
library(splancs)     # Spatial and Space-Time Point Pattern Analysis
library(spatstat)    # Spatial Point Pattern Analysis, Model-Fitting, Simulation, Tests


# Input:
# ---------------------------------------
nd <- c(25, 50, 75, 125, 150, 200, 250, 300, 400, 450, 500) # Expected total number of points
B  <- 500                                                   # Number of iterations in each case
alpha <- 10                                                 # Value of alpha for alpha-convex hull computation 
a <- 5; b <- 2                                              # Ellipse parameters
set.seed(1234)

# Output:
# ------------------------------------
# w  ------------- Elliptical window
# ar  ------------ Real volume of S
# lambdav  ------- Intensity
# n  ------------- Sample size

# T0 ------------- Minimax estimation of the volume
# T1 ------------- Gayraud estimator
# T2 ------------- UMVU oracle estimator
# T3 ------------- Plug-in MLE estimator
# T4 ------------- Baldin and Reib estimator
# T5 ------------- Naive estimator (Area of the convex hull)
# T6 ------------- Naive oracle estimator


w <- ellipse(a = a, b = b, centre = c(0, 0), phi = pi/6)
ar <- pi * a * b 
lambdav <- nd/ar
n <- lambdav * ar

nlam <- length(lambdav)
T0 <- T1 <- T2 <- T3 <- T4 <- T5 <- T6 <- matrix(nr = nlam, ncol = B)

for (i in 1:nlam) {
	lambda <- lambdav[i]
      for (j in 1:B) {
		ppp <- rpoispp(lambda, win = w) # Generate a random point pattern using the Poisson process
		N <- ppp$n # Sample size
		xN <- ppp$x
		yN <- ppp$y
    samp <- cbind(ppp$x, ppp$y)
		chat <- chull(samp)   # Convex hull of the sample
		Nd <- length(chat)    # Number of observed points on the boundary of the convex hull
		N0 <- N - Nd          # Number of observed points in the interior of the convex hull
		arch <- areapl(samp[chat, ]) # Area of the convex hull
		Nast <- floor(N/3)    # The sample is divided into three parts
		s1 <- samp[1:Nast, ]                     # Subsample 1
		s2 <- samp[(Nast + 1):(2 * Nast), ]      # Subsample 2
		s3 <- samp[(2 * Nast + 1):(3 * Nast), ]  # Subsample 3
		chat1 <- chull(s1)            # Convex hull of subsample 1
		arch1 <- areapl(s1[chat1, ])  # Area of the convex hull of subsample 1
		chat2 <- chull(s2)            # Convex hull of subsample 2
		arch2 <- areapl(s2[chat2, ])  # Area of the convex hull of subsample 2
		noin <- Nast - length(inpip(s3, s1[chat1, ])) # Points in subsample 3 outside convex hull of subsample 1

    T0[i, j] <- volest(samp, alpha, 1:floor(N*0.5))$vhat # Minimax estimation of the volume
    T1[i, j] <- arch1 + arch2/Nast * noin  # Gayraud estimator		
    T2[i, j] <- arch + Nd/lambda           # UMVU oracle estimator
		T3[i, j] <- arch * (1 + Nd/N)          # Plug-in MLE estimator
		T4[i, j] <- (N + 1)/(N0 + 1) * arch    # Baldin and Reib estimator
    T5[i, j] <- arch                       # Naive estimator (Area of the convex hull)
		T6[i, j] <- N/lambda                   # Naive oracle estimator
  	}
}



# Summary results 
# ------------------------------------

library(ggplot2)

# Figure 6 (left): RMSE of volume estimators
# --------------------------------------------
estim <- paste("T", 0:6, sep = "")
rmse <- stack(data.frame(sapply(estim, function(x) sqrt(rowMeans((eval(parse(text = x)) - ar)^2))/ar)))
rmse <- cbind(n, rmse)

ggplot(rmse, aes(x = n, y = values, linetype=ind,color=ind)) + geom_line(size = 1) 

# Figure 6 (right): RMSE of volume estimators With respect to the oracle estimator
# --------------------------------------------
windows()
mseo <- stack(data.frame(sapply(estim, function(x) rowMeans((eval(parse(text = x)) - ar)^2)/rowMeans((T6 - ar)^2))))
mseo <- cbind(n, mseo)

ggplot(mseo, aes(x = n, y = values, linetype=ind,color=ind)) + geom_line(size = 1) 


