# =============================================================================================
# Minimax Estimation of the Volume of a Set under the Rolling Ball Condition
# =============================================================================================

# Numerical experiments (Section 4)
# ---------------------------------------

# Non-convex case
# Performance of the optimal volume estimator described in Algorithm 1, bagging technique and confidence intervals


library(alphahull)   # Requires alphahull package (version 2.1)
source("volest.R")   # Read in R the function volest (minimax estimation of the volume)


# Input:
# ---------------------------------------
r <- 0.25; R <- 0.5                # Set S: disk with radii r and R
n <- 200                           # Sample size (n = 200, n = 500, n = 1000)
alphav <- c(0.15, 0.2, 0.25, 0.3)  # Values of alpha for alpha-convex hull computation 
B <- 500                           # Number of samples from the uniform distribution on S
b <- 1                             # Number of sample splittings for each one of the B uniform samples 
                                   # To replicate results in Figure 2, choose b = 1
                                   # To replicate results in Figure 3, choose b = 100
j <- 1:9                           # Points in the first subsample (m = [n*j/10])

# Output:
# ------------------------------------
# vr -----  Real volume of S
# x  -----  Samples generated from the uniform distribution on S (array with dimension B x n x 2) 
# nm -----  Length of vector j
# m  -----  Number of points in the first subsample (vector of length nm)
# wf  ----- Observations in the first subsample (matrix with dimension n x b)
#           wf[1:m[i], j] stores the indexes of the m[i] observations in the first subsample for splitting j (i in {1,...,nm}, j in {1,...,b})
# nalpha -- Length of alphav
# vah ----- Area of the alpha-convex hull of the whole sample (matrix with dimension B x nalpha)
#           vah[i,j] stores the area of the alpha-convex hull of sample i (sample with size n) for alpha=alphav[j] 
# vest ---- Volume estimation, see Algorithm 1 (array with dimension B x nalpha x b x nm)
#           vest[i,j,k,l] stores the volume estimation computed from sample i for alpha=alphav[j] and using as first subsample the observations wf[1:m[l], k]
# phat ---- Fraction of the second subsample that fall outside the alpha-convex hull of the first subsample (array with dimension B x nalpha x b x nm)
#           phat[i,j,k,l] is computed from sample i for alpha=alphav[j] and using as first subsample the observations wf[1:m[l], k]
# vfah ---- Area of the alpha-convex hull of the first subsample (array with dimension B x nalpha x b x nm)
#           vfah[i,j,k,l] stores the area of the alpha-convex hull for alpha=alphav[j] of the first subsample obtained from sample i with observations wf[1:m[l], k]


set.seed(1234)
vr <- pi * (R^2 - r^2)

# Uniform samples
theta <- runif(n * B, 0, 2 * pi)
ru <- sqrt(runif(n * B, r^2, R^2))
x <- cbind(ru * cos(theta), ru * sin(theta))
x <- array(x, dim = c(B, n, 2))

# Sample splittings
nm <- length(j)
m <- floor(n * j/10)
wf <- replicate(b, sample(n))


# Volume estimation
nalpha <- length(alphav)
vah <- matrix(nrow = B, ncol = nalpha)
vest <- array(dim = c(B, nalpha, b, nm))
phat <- array(dim = c(B, nalpha, b, nm))
vfah <- array(dim = c(B, nalpha, b, nm))

for (i1 in 1:B) {
	for (i2 in 1:nalpha) {
		alpha <- alphav[i2]
		vah[i1, i2] <- areaahull(ahull(x[i1, , ], alpha = alpha))
		for (i3 in 1:b) {
			for (i4 in 1:nm) {
			  aux <- volest(x[i1, , ], alpha, wf[1:m[i4], i3])
			  vest[i1, i2, i3, i4] <- aux[[1]]
			  phat[i1, i2, i3, i4] <- aux[[2]]
			  vfah[i1, i2, i3, i4] <- aux[[3]]
			}
		}
	}
}


# Summary results 
# ------------------------------------

# Figures 3-4: Erros bar plots (with base-10 log scale for the Y-axis)
# -------------------------------------------------------------------
aux <- abs(apply(vest, c(1, 2, 4), mean, na.rm = TRUE) - vr)/vr
mnj <- apply(aux, c(2, 3), mean, na.rm = TRUE)         # mean -- minimax volume estimator (j = 1:9)
sdj <- apply(aux, c(2, 3), sd, na.rm = TRUE)           # sd -- minimax volume estimator (j = 1:9)
mn10 <- apply(abs(vah - vr)/vr, 2, mean, na.rm = TRUE) # mean -- convex hull of the sample (j = 10)
sd10 <- apply(abs(vah - vr)/vr, 2, sd, na.rm = TRUE)   # sd -- convex hull of the sample (j = 10)
mn <- as.numeric(t(cbind(mnj, mn10)))
sd <- as.numeric(t(cbind(sdj, sd10)))
aux <- data.frame(alpha = rep(as.character(alphav), each = (nm+1)), j = rep(1:10, nalpha), mn, sd)

library(ggplot2)
ggplot(aux, aes(x = j, y = mn, colour = alpha)) +  
geom_errorbar(aes(ymin = mn - sd, ymax = mn + sd), size = 1) +  
geom_point(size = 3) + 
scale_x_continuous(name = "j", breaks = 1:10) +
scale_y_log10( )


# Table 1 
# Estimated coverage probability and estimated mean length of the confidence intervals
# for the volume computed with the method of Wilson (1927)  
# -------------------------------------------------------------------------------------
l <- 5  # Points of the first subsample m = m[l]
j <- 3  # Value of alpha (alpha = alphav[j])
qalphav <- seq(0.5, 0.05, by = -0.05) # Confidence level 1-qalpha
zqv <- qnorm(1 - qalphav * 0.5)
ncl <- length(qalphav)
m2 <- n - m[l]
phat1 <- phat[, j, 1, l]
llB <- matrix(nr = ncl, ncol = B) # Lower limits 
ulB <- matrix(nr = ncl, ncol = B) # Upper limits

for (i in 1:ncl) {
	zq <- zqv[i]
	aux1 <- 1/(1 + 1/m2 * zq^2)
	aux2 <- phat1 + 0.5/m2 * zq^2
	aux3 <- zq * sqrt(1/m2 * phat1 * (1 - phat1) + 0.25/m2^2 * zq^2)
	ul <- aux1 * (aux2 + aux3)
	ll <- aux1 * (aux2 - aux3)
	ulB[i, ] <- vfah[, j, 1, l]/(1 - ul)
	llB[i, ] <- vfah[, j, 1, l]/(1 - ll)
}

covW <- matrix(apply(llB < vr & ulB > vr, 1, mean), nr = 1)    # Coverage
tabicW <- rbind(1 - qalphav, covW, apply(ulB - llB, 1, mean))  # Coverage and mean length
row.names(tabicW) <- c("Level", "Cov", "Length")




